package org.seamcat.model;

import org.seamcat.model.simulation.EventProcessingResults;

import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

public class EventPluginResultsTableModel implements TableModel {

    private final int singleSize;
    private final int vectorSize;
    private final int scatterSize;
    private final int size;
    private EventProcessingResults result;

    public EventPluginResultsTableModel(EventProcessingResults result) {
        this.result = result;
        if ( result.failed() ) {
            singleSize = 0;
            vectorSize = 0;
            scatterSize = 0;
            size = 1;
        } else {
            singleSize = result.getResultTypes().getSingleValueTypes().size();
            vectorSize = result.getResultTypes().getVectorResultTypes().size();
            scatterSize = result.getResultTypes().getScatterDiagramResultTypes().size();
            size = singleSize + vectorSize + scatterSize ; 
        }
    }

    @Override
    public int getRowCount() {
        return size;
    }

    @Override
    public int getColumnCount() {
        return 4;
    }
    private final String[] COLS = new String[] {"Name", "Value", "Unit", "Type"};

    @Override
    public String getColumnName(int i) {
        return COLS[i];
    }

    public Class<?> getColumnClass(int columnIndex) {
		return String.class;
	}

    @Override
    public boolean isCellEditable(int i, int i2) {
        return false;
    }

    @Override
    public Object getValueAt(int row, int col) {
        if ( result.failed() ) {
            if ( col == 0 ) return result.getName();
            if ( col == 1 ) return result.getException();
            if ( col == 2 ) return "N/A";
            return "Exception";
        }
        if ( row < singleSize ) {
            if ( col == 0 ) return result.getResultTypes().getSingleValueTypes().get(row).getName();
            if ( col == 1 ) return result.getResultTypes().getSingleValueTypes().get(row).getValue();
            if ( col == 2) return result.getResultTypes().getSingleValueTypes().get(row).getUnit();
            //TODO changed by Karl Koch and proposed to STG
            if(result.getResultTypes().getSingleValueTypes().get(row).getValue().getClass().getName().contains("String"))return "String";
            else if (result.getResultTypes().getSingleValueTypes().get(row).getValue().getClass().getName().contains("Integer"))return "Integer";
            else return "Double";
        } else if ( row < vectorSize + singleSize) {
            if ( col == 0 ) return result.getResultTypes().getVectorResultTypes().get(row-singleSize).getName();
            if ( col == 1 ) return result.getResultTypes().getVectorResultTypes().get(row-singleSize).getValue();
            if ( col == 2 ) return result.getResultTypes().getVectorResultTypes().get(row-singleSize).getUnit();
            return "Vector";
        } else {
            if ( col == 0 ) return result.getResultTypes().getScatterDiagramResultTypes().get(row-vectorSize - singleSize).getTitle(); 
            if ( col == 1 ) return result.getResultTypes().getScatterDiagramResultTypes().get(row-vectorSize - singleSize); 
            if ( col == 2 ) return "NA";
            return "Scatter";
        }
    }

    @Override
    public void setValueAt(Object o, int i, int i2) {
        throw new UnsupportedOperationException();
    }

    @Override
    public void addTableModelListener(TableModelListener tableModelListener) {
    }

    @Override
    public void removeTableModelListener(TableModelListener tableModelListener) {
    }

	@Override
	public String toString() {
		return "Results for plugin '"+result.getName()+"'";
	}
}
