package org.seamcat.epp;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.swing.JFrame;

import org.seamcat.model.functions.Point2D;
import org.seamcat.model.plugin.Parameter;

/**
 * This class acts as assistant doing the calculations requested by the external
 * frame<br/>
 * It acts also as the connection to the main part of the EPP by getting the
 * collected vectors / parameters<br/>
 * and providing the calculated results by synchronised getters
 * 
 * @author karl
 * 
 */
public class TestFrameCalc extends TestFrame {

	private JFrame frame;
	private List<Parameter> params;
	private List<Double> dRSS;
	private boolean isPanelClosed;
	private List<Double> result1;
	private List<Double> result2;
	private List<Double> result3;
	private boolean showResult1;
	private boolean showResult2;
	private boolean showResult3;
	private boolean isResultAvailable;
	private boolean[] selectedParams;
	private List<Double> iRSSunw;
	private double CI;
	private List<Double> distibutionCI;
	private List<Point2D> functionCI;
	private double propbabilityCI = -100;
	private double sensitivity;

	public TestFrameCalc() {
	}

	public void setFrame(JFrame jFrame) {
		this.frame = jFrame;
	}

	public void setParams(List<Parameter> params) {
		this.params = params;
	}

	public void setVectors(List<Double> dRSS, List<Double> iRSSunw, double tCI, double sensitivity) {
		this.dRSS = dRSS;
		this.iRSSunw = iRSSunw;
		this.CI = tCI;
		this.sensitivity = sensitivity;
		result1 = new ArrayList<Double>();
		result2 = new ArrayList<Double>();
		result3 = new ArrayList<Double>();
	}

	public void initializeResultPanel() {
		setCalculator(this);

		getFrame().setVisible(true);

	}

	public synchronized boolean getIsPanelClosed() {
		return isPanelClosed;
	}

	public TestFrameCalc getCalculator() {
		return this;
	}

	public synchronized boolean getResultsAvailable() {
		return isResultAvailable;
	}

	/**
	 * controls the requested calculations
	 */
	public synchronized void calculateResults() {
		selectedParams = getSelectedParams();
		showResult1 = selectedParams[0];
		showResult2 = selectedParams[1];
		showResult3 = selectedParams[2];
		double correction = 0.;
		if (showResult1) {
			for (int i = 0; i < dRSS.size(); i++) {
				result1.add(dRSS.get(i) * getSelectedFactor() + correction);
			}
		}
		if (showResult2) {
			correction = 10.;
			for (int i = 0; i < dRSS.size(); i++) {
				result2.add(dRSS.get(i) * getSelectedFactor() + correction);
			}
		}
		if (showResult3) {
			correction = 20.;
			for (int i = 0; i < dRSS.size(); i++) {
				result3.add(dRSS.get(i) * getSelectedFactor() + correction);
			}
		}
		distibutionCI = new ArrayList<Double>();
		calculateCI();
		setExceedingLimit();
		setFunctionCI();

	}

	/**
	 * prepares the scattered diagram
	 */
	private void setFunctionCI() {
		functionCI = new ArrayList<Point2D>();
		double[][] sortedCI = new double[dRSS.size()][2];
		sortedCI = getSortedDataAsCDF(distibutionCI);
		for (int i = 0; i < distibutionCI.size(); i++) {
			double x = sortedCI[i][0];
			double y = sortedCI[i][1];
			functionCI.add(new Point2D(sortedCI[i][0], sortedCI[i][1]));
		}
	}

	private void calculateCI() {
		for (int i = 0; i < dRSS.size(); i++) {
			double rCI = dRSS.get(i) - iRSSunw.get(i);
			distibutionCI.add(rCI);
		}
	}

	public synchronized List<Point2D> getDistributionCI() {
		return functionCI;
	}

	public synchronized List<Double> getDistributionCIvalues() {
		return distibutionCI;
	}

	/**
	 * prepares the single result of exceeding the limit<br/>
	 * this is not completely in line with SEAMCAT as it considers only dRSS
	 * values greater than sensitivity
	 */
	private void setExceedingLimit() {
		int n = 0;
		for (int i = 0; i < dRSS.size(); i++) {
			if (distibutionCI.get(i) < CI) {
				n++;
			}
		}
		propbabilityCI = (double) (n - getBelowSens()) / dRSS.size() * 100; // in percent
	}

	private int getBelowSens() {
		int n = 0;
		for (int i = 0; i < dRSS.size(); i++) {
			if (dRSS.get(i) < sensitivity) {
				n++;
			}
		}
		return n;
	}

	public synchronized double getExceedingLimit() {
		return propbabilityCI;
	}

	public synchronized String getResultText() {
		return "some text about calculated results: here";
	}

	public synchronized List<Double> getResultVector1() {
		return result1;
	}

	public synchronized List<Double> getResultVector2() {
		return result2;
	}

	public synchronized List<Double> getResultVector3() {
		return result3;
	}

	public synchronized boolean getReadyForExport() {
		if (showResult1 || showResult2 || showResult3) {
			setResultsCalculated(true);
			return true;
		}
		return false;
	}

	public synchronized void setPanelClosed(boolean flag) {
		isPanelClosed = flag;
	}

	public synchronized void setResultsCalculated(boolean flag) {
		isResultAvailable = flag;
	}

	/**
	 * creates a CDF from a vector
	 * 
	 * @param localList
	 * @return the List<Doulbe> as sorted double[events][2] with the ascending
	 *         probabilities on the second column
	 */
	public double[][] getSortedDataAsCDF(List<Double> localList) {
		double[][] sorted = new double[localList.size()][2];
		double[] sortedData = new double[localList.size()];
		for (int i = 0; i < localList.size(); i++) {
			sortedData[i] = localList.get(i);
		}
		Arrays.sort(sortedData);
		for (int j = 0; j < sortedData.length; j++) {
			sorted[j][0] = sortedData[j];
			sorted[j][1] = new Double((double) j / (double) sortedData.length);
		}
		return sorted;
	}
	
	public synchronized String getSettings(){
		String msg = "";
		msg = "factor for dRSS: " + getSelectedFactor() ;
		msg += " *-*-* constants used: ";
		if(showResult1) msg += "0   ";
		if(showResult2) msg += "+10   ";
		if(showResult3) msg += "+20";
		return msg;
	}

	/**
	 * in case "close automatically" is selected
	 */
	public synchronized void closeExternalFrame() {
		getFrame().dispose();
	}
}
